/*
  craft.c

  For C8zy.  Mapping of crafted items to their results.
  Bill Kendrick <bill@newbreedsoftware.com>

  2025-03-01 - 2025-03-02
*/

#include "craft.h"
#include "map.h"
#include "player.h"
#include "items.h"
#include "shapes.h"

/* Combining up to four items on hand (ITEM_...), to create a new item (ITEM_...) */
unsigned char craft_onhand[][CRAFT_ITEM_CT + 1] = {
    {ITEM_DIRT,    ITEM_WATER,   ITEM_NONE,    ITEM_NONE,    ITEM_MUD},
    {ITEM_GRASS,   ITEM_GRASS,   ITEM_GRASS,   ITEM_NONE,    ITEM_ROPE},
    {ITEM_PLANK,   ITEM_PLANK,   ITEM_PLANK,   ITEM_ROPE,    ITEM_BEAMS},
    {ITEM_LOG,     ITEM_ROPE,    ITEM_ROCK,    ITEM_NONE,    ITEM_HAMMER},
    {ITEM_LOG,     ITEM_ROPE,    ITEM_GEM_5,   ITEM_NONE,    ITEM_AX},
    {ITEM_FLOUR,   ITEM_WATER,   ITEM_NONE,    ITEM_NONE,    ITEM_DOUGH},

    {ITEM_NONE,    ITEM_NONE,    ITEM_NONE,    ITEM_NONE,    ITEM_NONE}, /* marks end of list */
};

/* Combining an item from inventory onto a map object, to get a new item */
/* For each pair of:
 *   0. the item from inventory, being combined with...  (ITEM_...)
 *   1. ...the shape on the map                          (SH...)
 * we have:
 *   2. what item is gained into inventory, and...       (ITEM_...)
 *   3. what shape remains on the map                    (SH_...)
 */
unsigned char craft_onland[][4] = {
    {ITEM_MUD,       SH_FIRE,       ITEM_BRICK,     SH_FIRE}, /* mud + fire = brick */
    {ITEM_DIRT,      SH_WATER_SHAL, ITEM_MUD,       SH_WATER_SHAL}, /* dirt + water = mud */
    {ITEM_DIRT,      SH_WATER_DEEP, ITEM_MUD,       SH_WATER_DEEP}, /* dirt + water = mud */
    {ITEM_WATER,     SH_DIRT,       ITEM_MUD,       SH_DIRT}, /* dirt + water = mud */
    {ITEM_CRAB,      SH_FIRE,       ITEM_CRAB_COOK, SH_FIRE}, /* crab cooks on the fire */
    {ITEM_FISH,      SH_FIRE,       ITEM_FISH_COOK, SH_FIRE}, /* fish cooks on the fire */
    {ITEM_FISH_COOK, SH_FIRE,       ITEM_FISH_BONE, SH_FIRE}, /* cooked fish burns to bones */
    {ITEM_WATER,     SH_FIRE,       ITEM_NONE,      SH_DIRT}, /* water puts out fire */
    {ITEM_SAND,      SH_FIRE,       ITEM_GLASS,     SH_DIRT}, /* sand puts out fire & makes glass */
    {ITEM_BEAMS,     SH_WATER_SHAL, ITEM_NONE,      SH_BRIDGE}, /* beams on water become a bridge */
    {ITEM_BEAMS,     SH_WATER_DEEP, ITEM_NONE,      SH_BRIDGE}, /* beams on water become a bridge */
    {ITEM_BEAMS,     SH_DIRT,       ITEM_NONE,      SH_FENCE}, /* beams on land becomes a fence */
    {ITEM_BEAMS,     SH_SAND,       ITEM_NONE,      SH_FENCE}, /* beams on land becomes a fence */
    {ITEM_BEAMS,     SH_GRASS,      ITEM_NONE,      SH_FENCE}, /* beams on land becomes a fence */
    {ITEM_DOUGH,     SH_FIRE,       ITEM_BREAD,     SH_FIRE}, /* dough turns to bread on the fire */
    {ITEM_SEEDS_A,   SH_MOUND,      ITEM_NONE,      SH_SPROUT /* FIXME */}, /* plant seeds into mound */
    {ITEM_SEEDS_B,   SH_MOUND,      ITEM_NONE,      SH_SPROUT /* FIXME */}, /* plant seeds into mound */
    {ITEM_WATER,     SH_SPROUT,     ITEM_NONE,      SH_FLOWER1_A /* FIXME */}, /* water a sprout & it grows */

    {ITEM_NONE,      SH_DIRT,       ITEM_NONE,      SH_DIRT}, /* marks end of list */
};

int place(unsigned char item, int wx, int wy) {
  int i, found;
  int wyy;

  wyy = wy * W;

  if (map[wyy + wx] == SH_DIRT) {
    /* Dirt? Just place it on top (if we can) */
    found = -1;
    for (i = 0; droppable_items[i][0] != ITEM_NONE && found == -1; i++) {
      if (droppable_items[i][0] == items[item]) {
        found = i;
      }
    }

    if (found != -1) {
      map[wyy + wx] = droppable_items[found][1];
      items[item] = ITEM_NONE;
      return(PLACE_DROP);
    }
  }

  /* See if it's something we can 'craft onto the ground'? */
  found = -1;
  for (i = 0; craft_onland[i][0] != ITEM_NONE && found == -1; i++) {
    if (craft_onland[i][0] == items[item] && craft_onland[i][1] == map[wyy + wx]) {
      found = i;
    }
  }

  if (found == -1) {
    return(PLACE_BAD);
  }

  items[item] = craft_onland[found][2];
  map[wyy + wx] = craft_onland[found][3];
  tired = tired + TIRED_DROP;

  return(items[item]);
}

int craft(unsigned char craft_item_idx[], unsigned char num_craft_items) {
  int i, recipe_match, min_item_slot;

  recipe_match = -1;

  /* Search for a matching recipe... */
  for (i = 0; craft_onhand[i][0] != ITEM_NONE && recipe_match == -1; i++) {
    int items_in_recipe, j;

    /* Count how many items in this recipe */
    items_in_recipe = 0;
    for (j = 0; j < CRAFT_ITEM_CT && craft_onhand[i][j] != ITEM_NONE; j++) {
      items_in_recipe++;
    }

    if (items_in_recipe == num_craft_items) {
      /* Same number of items in recipe as we want to craft; proceed */
      int recipe_hit[CRAFT_ITEM_CT];
      int num_matches;

      /* Check that every item we're trying to craft matches one item
         in the recipe (it doesn't matter what order they're in) */
      for (j = 0; j < CRAFT_ITEM_CT; j++) {
        recipe_hit[j] = 0;
      }

      num_matches = 0;
      for (j = 0; j < num_craft_items; j++) {
        int k, matched;

        matched = 0;
        for (k = 0; k < items_in_recipe && matched == 0; k++) {
          if (recipe_hit[k] == 0 && craft_onhand[i][k] == items[craft_item_idx[j]]) {
            recipe_hit[k] = 1;
            matched = 1;
            num_matches++;
          }
        }
      }

      if (num_matches == items_in_recipe) {
        /* Exact match! */
        recipe_match = i;
      }
    }
  }

  if (recipe_match == -1) {
    /* What we're trying to craft didn't match any recipes */
    return(CRAFT_BAD);
  }

  /* Craft the items !*/

  /* Remove each item being used to craft (and take note of
     the earliest slot any of them came from, so we can put
     the new item there) */
  min_item_slot = MAX_ITEMS;
  for (i = 0; i < num_craft_items; i++) {
    items[craft_item_idx[i]] = ITEM_NONE;
    if (craft_item_idx[i] < min_item_slot) {
      min_item_slot = craft_item_idx[i];
    }
  }

  /* Put in the new item! */
  items[min_item_slot] = craft_onhand[recipe_match][CRAFT_ITEM_CT];

  return(items[min_item_slot]);
}

