/*
  io_atari.c

  For C8zy.  I/O for Atari.
  Bill Kendrick <bill@newbreedsoftware.com>

  (Some draw_*() functions cribbed from Firefighter, 2023-2025)

  2025-03-09 - 2025-03-09
*/

#include <stdio.h>
#include <string.h>
#include <atari.h>
#include <peekpoke.h>
#include "io.h"
#include "player.h"
#include "items.h"
#include "map.h"
#include "shapes.h"
#include "craft.h"

extern unsigned char scr_mem[];
extern unsigned char tilegfx[];
unsigned char * dlist = scr_mem + 1024;

void * OLDVEC;
char msg[41];

void dli1(void);
void dli2(void);


/* Draw some text on the screen.

   @param char * str - The NUL-terminated ('\0') string to write
   @param unsigned char * dest - The destination in memory
     (expected to be within scr_mem[]!!!)
*/
void draw_text(char * str, unsigned char * dest) {
  unsigned char ch, ch2;
  unsigned int i;

  for (i = 0; str[i] != '\0'; i++) {
    ch = str[i];
    ch2 = ch & 127;

    if (ch2 < 32) {
      ch |= 64;
    } else if (ch2 < 96) {
      ch = ch - 32;
    }

    *dest = ch;
    dest++;
  }
}

/* Draw a zero-padded decimal number on the screen.

   @param unsigned long int n - The number to draw
   @param signed char digits - How many digits to show (will be zero-padded)
   @unsigned char * dest - The destination in memory
     (expected to within scr_mem[]!!!)
*/
void draw_number(unsigned long int n, signed char digits, unsigned char * dest) {
  unsigned char * ddest;
  ddest = dest + digits - 1;
  do {
    *ddest = (n % 10) | 16;

    digits--;
    if (digits) {
      n = (n * 0xCCCD) >> 19; /* n = n / 10 */
      ddest--;
    }
  } while (digits);
}

void dli1(void) {
  asm("pha");

  asm("sta $D40A"); // WSYNC

  /* Set color palette */
  asm("lda $603");
  asm("lda #$ff");
  asm("sta %w", (unsigned)&GTIA_WRITE.colbk);
  asm("lda $604");
  asm("sta %w", (unsigned)&GTIA_WRITE.colpf0);
  asm("lda $605");
  asm("sta %w", (unsigned)&GTIA_WRITE.colpf1);
  asm("lda $606");
  asm("sta %w", (unsigned)&GTIA_WRITE.colpf2);
  asm("lda $607");
  asm("sta %w", (unsigned)&GTIA_WRITE.colpf3);

  asm("lda # <%v", dli2);
  asm("sta %w", (unsigned)&OS.vdslst);
  asm("lda # >%v", dli2);
  asm("sta %w", ((unsigned)&OS.vdslst) + 1);

  asm("lda $600");
  asm("sta $D409"); // ==> CHBASE

  asm("pla");
  asm("rti");
}

void dli2(void) {
  asm("pha");

  /* Flip which half of the font we're showing */
  asm("lda #$04");
  asm("sec");
  asm("sbc $601");
  asm("sta $601");

  asm("clc");
  asm("adc $600");
  asm("adc $602"); // FIXME for animation

  asm("sta $D409"); // ==> CHBASE

  asm("sta $D40A"); // WSYNC

  asm("pla");
  asm("rti");
}

void dli_vbi(void) {
  /* Start display with the first DLI */
  asm("lda # <%v", dli1);
  asm("sta %w", (unsigned)&OS.vdslst);
  asm("lda # >%v", dli1);
  asm("sta %w", ((unsigned)&OS.vdslst) + 1);

  asm("lda #0");
  asm("sta $604");

  asm("jmp (%v)", OLDVEC);
}

/* Set up display list */
void setup_displaylist(void) {
  unsigned char i;
  unsigned char * ptr, * scr_ptr;

  ptr = dlist;

  /* Blanks at the top */
  *(ptr++) = DL_BLK8;
  *(ptr++) = DL_BLK4;
  *(ptr++) = DL_BLK8;

  /* One line of text at the top */
  *(ptr++) = DL_LMS(DL_GRAPHICS0);
  POKEW(ptr, (int) scr_mem);
  ptr += 2;

  *(ptr++) = DL_BLK1;

  for (i = 0; i < 4; i++) {
    *(ptr++) = DL_GRAPHICS0;
  }

  *(ptr++) = DL_BLK2;

  for (i = 0; i < 3; i++) {
    *(ptr++) = DL_GRAPHICS0;
  }
    *(ptr++) = DL_DLI(DL_GRAPHICS0);

  *(ptr++) = /*DL_DLI*/(DL_BLK1);

  scr_ptr = scr_mem;
  scr_ptr += ((24 - H) * 40);

  for (i = 0; i < H; i++) {
    *(ptr++) = DL_DLI(DL_LMS(DL_GRAPHICS12));
    POKEW(ptr, (int) scr_ptr);
    ptr += 2;

    *(ptr++) = DL_DLI(DL_LMS(DL_GRAPHICS12));
    POKEW(ptr, (int) scr_ptr);
    ptr += 2;

    scr_ptr += 40;
  }

  /* Jump back to the top of this Display List */
  *(ptr++) = DL_JVB;
  POKEW(ptr, (int) dlist);

  /* Enable our Display List */
  OS.sdlst = dlist;

  POKE(0x600, (unsigned char) ((unsigned int) tilegfx / 256));
  POKE(0x601, 0);

  /* Activate Vertical Blank Interrupt */
  OLDVEC = OS.vvblkd;
  OS.critic = 1;
  OS.vvblkd = (void *) dli_vbi;
  OS.critic = 0;

  /* Activate Display List Interrupt */
  ANTIC.nmien = NMIEN_VBI;
  while (ANTIC.vcount < 124);
  OS.vdslst = (void *) dli1;
  ANTIC.nmien = NMIEN_VBI | NMIEN_DLI;
}

void set_palette(unsigned char c0, unsigned char c1, unsigned char c2, unsigned char c3, unsigned char c4) {
  POKE(0x603, c0);
  POKE(0x604, c1);
  POKE(0x605, c2);
  POKE(0x606, c3);
  POKE(0x607, c4);
}

/* Initialize things at the beginning of the program */
void io_init(void) {
  /* FIXME: Eventually set up display list, static elements, etc. */

  /* Clear screen */
  memset(scr_mem, 0, 960);

  set_palette(0x1C, 0x00, 0xC8, 0x44, 0x88);
  setup_displaylist();

  /* Stats section at the top */
  sprintf(msg, "Hunger: __ / Thirst: __ / Tired: __");
  draw_text(msg, scr_mem + 3);
}

/* Draw player stats */
void io_draw_stats(void) {
  draw_number(hunger, 2, scr_mem + 3 + 8);
  draw_number(thirst, 2, scr_mem + 3 + 21);
  draw_number(tired, 2, scr_mem + 3 + 33);
}

/* Show item inventory
   @param selected[] - NULL | array of booleans,
     one for each indexes into the player's inventory,
     denoting which one(s) (if any) is/are selected
*/
void io_draw_inventory(unsigned char selected[]) {
  unsigned char i, j, is_selected;
  unsigned char * scr_ptr;

  scr_ptr = scr_mem + (1 * 40);

  for (i = 0; i < MAX_ITEMS; i++) {
    is_selected = 0;
    if (selected != NULL) {
      for (j = 0; j < CRAFT_ITEM_CT && is_selected == 0; j++) {
        if (selected[j] == i) {
          is_selected = 1;
        }
      }
    }

    if (is_selected) {
      sprintf(msg, ">>> %-6s", item_names[items[i]]);
    } else {
      sprintf(msg, "%2d. %-6s", i + 1, item_names[items[i]]);
    }
    draw_text(msg, scr_ptr);
    scr_ptr += 10;
  }
}

/* Draw the map */
void io_draw_map(void) {
  unsigned char x, y;
  unsigned char * scr_ptr;
  int yy;

  scr_ptr = scr_mem;
  scr_ptr += ((24 - H) * 40);
  scr_ptr += 4;

  yy = 0;

  for (y = 0; y < H; y++) {
    for (x = 0; x < W; x++) {
      if (px == x && py == y) {
        *(scr_ptr++) = SH_YOU;
        *(scr_ptr++) = SH_YOU + 1;
      } else {
        *(scr_ptr++) = map[yy + x];
        *(scr_ptr++) = map[yy + x] + 1;
      }
    }

    scr_ptr += 8;
    yy += W;
  }
}

unsigned char msg_pos = 0;

/* Show a message
   @param char * msg - the message to display */
void io_show_msg(char * msg) {
  memset(scr_mem + (7 + msg_pos) * 40, 0, 40);
  draw_text(msg, scr_mem + (7 + msg_pos) * 40);
  msg_pos = 1 - msg_pos;
}

void zap_prompt(void) {
  memset(scr_mem + 5 * 40, 0, 80);
}

/* Prompt for, and get a direction -- actually, a position
   relative to the player's current position -- from the user.
   @param char * prompt - a prompt for the user
   @param int * wx, int * wy - the x,y coordinates chosen
*/
/* FIXME - Convert to using joystick */
void io_get_dir(char * prompt, int * wx, int * wy)
{
  char c;

  zap_prompt();
  sprintf(msg, "%s (wasd=direction, x=under)", prompt);
  draw_text(msg, scr_mem + 5 * 40);

  *wx = -1;
  *wy = -1;

  scanf(" %c", &c);
  if (c == 'w') {
    *wx = px;
    *wy = py - 1;
  } else if (c == 'a') {
    *wx = px - 1;
    *wy = py;
  } else if (c == 's') {
    *wx = px;
    *wy = py + 1;
  } else if (c == 'd') {
    *wx = px + 1;
    *wy = py;
  } else if (c == 'x') {
    *wx = px;
    *wy = py;
  }
}

/* Receive control input from the player.
   @return unsigned char a CMD_... enumeration
*/    
/* FIXME - Convert to using joystick */
unsigned char io_get_cmd(void) {
  char c;

  // FIXME
  zap_prompt();
  draw_text("Cmd? (wasd=move g=gather p=place/drop", scr_mem + 5 * 40);
  draw_text("      u=use, c=craft, e=eat, q=quit)", scr_mem + 6 * 40);

  scanf(" %c", &c);

  switch(c) {
    case 'w':
      return CMD_DIR_UP;
      break;
    case 'd':
      return CMD_DIR_RIGHT;
      break;
    case 's':
      return CMD_DIR_DOWN;
      break;
    case 'a':
      return CMD_DIR_LEFT;
      break;
    case 'g':
      return CMD_GATHER;
      break;
    case 'p':
      return CMD_PLACE;
      break;
    case 'u':
      return CMD_USE;
      break;
    case 'c':
      return CMD_CRAFT;
      break;
    case 'e':
      return CMD_EAT;
      break;
    case 'q':
      return CMD_QUIT;
      break;
    default:
      return CMD_NONE;
      break;
  }
}

/* Prompt for user to choose an item, and return the chosen one
   (as an index into their `items[]` inventory array)
   @param char * prompt - a prompt for the user
   @return unsigned char - the index of the chosen item,
     or ITEM_BAD if none chosen
*/
/* FIXME - Convert to using joystick */
unsigned char io_get_item_idx(char * prompt) {
  int item;

  zap_prompt();
  sprintf(msg, "%s which item?", prompt);
  draw_text(msg, scr_mem + 5 * 40);

  scanf(" %d", &item);

  if (item < 1 || item > MAX_ITEMS) {
    return ITEM_BAD;
  }

  return item - 1;
}

/* Prompt for user to OPTIONALLY choose an item, and return the chosen
   one (as an index into their `items[]` inventory array),
   or ITEM_STOP if they are done.
   @param char * prompt - a prompt for the user
   @return unsigned char - the index of the chosen item,
     or ITEM_STOP if the user is done, or ITEM_BAD if none chosen
*/
/* FIXME - Convert to using joystick */
unsigned char io_get_craft_item_idx(char * prompt) {
  int item;

  zap_prompt();
  sprintf(msg, "%s which item? (0=stop)", prompt);
  draw_text(msg, scr_mem + 5 * 40);

  scanf(" %d", &item);

  if (item == 0) {
    return ITEM_STOP;
  } else if (item < 1 || item > MAX_ITEMS) {
    return ITEM_BAD;
  } else {
    return (item - 1);
  }
}
