/*
  io_atari.c

  For C8zy.  I/O for Atari.
  Bill Kendrick <bill@newbreedsoftware.com>

  2025-03-09 - 2025-03-09
*/

#include <stdio.h>
#include <string.h>
#include "io.h"
#include "player.h"
#include "items.h"
#include "map.h"
#include "shapes.h"
#include "craft.h"

/* Map shape bytes to ASCII character representations */
unsigned char sh_ascii[][2] = {
  { SH_DIRT, '_' },
  { SH_SAND, ':' },
  { SH_WATER_SHAL, '-' },
  { SH_WATER_DEEP, '^' },
  { SH_GRASS, 'v' },
  { SH_MOUND ,'m' },
  { SH_SPROUT, 'd' },
  { SH_FLOWER1_A, 'x' },
  { SH_FLOWER1_B, 'x' },
  { SH_FLOWER2_A, 'x' },
  { SH_FLOWER2_B, 'x' },
  { SH_MELON_A, 'Q' },
  { SH_MELON_B, 'Q' },
  { SH_SEEDS_A, ';' },
  { SH_SEEDS_B, ';' },
  { SH_BRICK, '#' },
  { SH_PLANK, '|' },
  { SH_WINDOW, '/' },
  { SH_DOOR, '[' },
  { SH_FIRE, '%' },
  { SH_LOG, 'o' },
  { SH_FENCE, '=' },
  { SH_BRIDGE, 'B' },
  { SH_GEM_1, '$' },
  { SH_GEM_2, '$' },
  { SH_GEM_3, '$' },
  { SH_GEM_4, '$' },
  { SH_GEM_5, '$' },
  { SH_ROCK_SM, '.' },
  { SH_ROCK_LG, 'R' },
  { SH_TREE, 'T' },
  { SH_TREE_BARE, 't' },
  { SH_TREE_BURNT, 'z' },
  { SH_TREE_FRUIT, 'F' },
  { SH_LEAVES, '&' },
  { SH_CRAB, 'X' },
  { SH_CRAB_COOK, 'y' },
  { SH_FISH, 'F' },
  { SH_FISH_COOK, 'f' },
  { SH_FISH_BONE, '>' },
  { SH_SNAIL, '@' },
  { SH_SHELL, 'C' },
  { SH_BREAD, 'b' },
  { SH_YOU, '+' },
  { 255, '_' },
};

unsigned char sh_ascii_map[255];

/* Initialize things at the beginning of the program */
void io_init(void) {
  unsigned char i;

  /* FIXME: Goes away eventually */
  for (i = 0; sh_ascii[i][0] != 255; i++) {
    sh_ascii_map[sh_ascii[i][0]] = sh_ascii[i][1];
  }

  /* FIXME: Eventually set up display list, static elements, etc. */
}

/* Draw player stats */
void io_draw_stats(void) {
  printf("\n\n");

  /* Show stats & map */
  printf("Hunger: %d / Thirst: %d / Tired: %d\n", hunger, thirst, tired);
}

/* Show item inventory
   @param unsigned char selected[] - NULL | array of booleans,
     one for each indexes into the player's inventory,
     denoting which one(s) (if any) is/are selected
*/
void io_draw_inventory(unsigned char selected[]) {
  unsigned char i;

  if (selected == NULL) {
    for (i = 0; i < MAX_ITEMS; i++) {
      printf("%2d. %-6s", i + 1, item_names[items[i]]);
    }
  } else {
    unsigned char j, is_selected;

    for (i = 0; i < MAX_ITEMS; i++) {
      is_selected = 0;
      for (j = 0; j < CRAFT_ITEM_CT && is_selected == 0; j++) {
        if (selected[j] == i) {
          is_selected = 1;
        }
      }

      if (is_selected) {
        printf(">>> %-6s", item_names[items[i]]);
      } else {
        printf("%2d. %-6s", i + 1, item_names[items[i]]);
      }
    }
  }
}

/* Draw the map */
void io_draw_map(void) {
  unsigned char x, y;
  int yy;

  printf("\n");

  for (y = 0; y < H; y++) {
    printf("%2d -> ", y);

    yy = y * W;
    for (x = 0; x < W; x++) {
      if (px == x && py == y) {
        printf("%c", sh_ascii_map[SH_YOU]);
      } else {
        printf("%c", sh_ascii_map[map[yy + x]]);
      }
    }
    printf("\n");
  }
}

/* Show a message
   @param char * msg - the message to display */
void io_show_msg(char * msg) {
  printf("%s\n", msg);
}

/* Prompt for, and get a direction -- actually, a position
   relative to the player's current position -- from the user.
   @param char * prompt - a prompt for the user
   @param int * wx, int * wy - the x,y coordinates chosen
*/
void io_get_dir(char * prompt, int * wx, int * wy)
{
  char c;

  printf("%s (wasd=direction, x=under)\n", prompt);

  *wx = -1;
  *wy = -1;

  scanf(" %c", &c);
  if (c == 'w') {
    *wx = px;
    *wy = py - 1;
  } else if (c == 'a') {
    *wx = px - 1;
    *wy = py;
  } else if (c == 's') {
    *wx = px;
    *wy = py + 1;
  } else if (c == 'd') {
    *wx = px + 1;
    *wy = py;
  } else if (c == 'x') {
    *wx = px;
    *wy = py;
  }
}

/* Receive control input from the player.
   @return unsigned char a CMD_... enumeration
*/
unsigned char io_get_cmd(void) {
  char c;

  printf("Cmd? (wasd=move g=gather p=place/drop\n");
  printf("      u=use, c=craft, e=eat, q=quit)\n");
  scanf(" %c", &c);

  switch(c) {
    case 'w':
      return CMD_DIR_UP;
      break;
    case 'd':
      return CMD_DIR_RIGHT;
      break;
    case 's':
      return CMD_DIR_DOWN;
      break;
    case 'a':
      return CMD_DIR_LEFT;
      break;
    case 'g':
      return CMD_GATHER;
      break;
    case 'p':
      return CMD_PLACE;
      break;
    case 'u':
      return CMD_USE;
      break;
    case 'c':
      return CMD_CRAFT;
      break;
    case 'e':
      return CMD_EAT;
      break;
    case 'q':
      return CMD_QUIT;
      break;
    default:
      return CMD_NONE;
      break;
  }
}

/* Prompt for user to choose an item, and return the chosen one
   (as an index into their `items[]` inventory array)
   @param char * prompt - a prompt for the user
   @return unsigned char - the index of the chosen item,
     or ITEM_BAD if none chosen
*/
unsigned char io_get_item_idx(char * prompt) {
  int item;

  printf("%s which item?\n", prompt);

  scanf(" %d", &item);

  if (item < 1 || item > MAX_ITEMS) {
    return ITEM_BAD;
  }

  return items[item - 1];
}

/* Prompt for user to OPTIONALLY choose an item, and return the chosen
   one (as an index into their `items[]` inventory array),
   or ITEM_STOP if they are done.
   @param char * prompt - a prompt for the user
   @return unsigned char - the index of the chosen item,
     or ITEM_STOP if the user is done, or ITEM_BAD if none chosen
*/
unsigned char io_get_craft_item_idx(char * prompt) {
  int item;

  printf("%s which item? (0=stop)\n", prompt);

  scanf(" %d", &item);

  if (item == 0) {
    return ITEM_STOP;
  } else if (item < 1 || item > MAX_ITEMS) {
    return ITEM_BAD;
  } else {
    return (item - 1);
  }
}
